// Copyright 2023 Georges Racinet <georges.racinet@octobus.net>
//
// This software may be used and distributed according to the terms of the
// GNU General Public License version 2 or any later version.
// SPDX-License-Identifier: GPL-2.0-or-later
use std::fmt::Debug;
use std::sync::Arc;

use tokio_util::sync::CancellationToken;
use tonic::{Request, Response, Status};

use crate::config::Config;
use crate::gitaly::operation_service_client::OperationServiceClient;
use crate::gitaly::operation_service_server::{OperationService, OperationServiceServer};
use crate::gitaly::{
    UserFfBranchRequest, UserFfBranchResponse, UserSquashRequest, UserSquashResponse,
};
use crate::sidecar;

#[derive(Debug)]
#[allow(dead_code)]
pub struct OperationsServiceImpl {
    config: Arc<Config>,
    shutdown_token: CancellationToken,
    sidecar_servers: Arc<sidecar::Servers>,
}

#[tonic::async_trait]
impl OperationService for OperationsServiceImpl {
    async fn user_squash(
        &self,
        request: Request<UserSquashRequest>,
    ) -> Result<Response<UserSquashResponse>, Status> {
        sidecar::unary!(self, request, OperationServiceClient, user_squash)
    }

    async fn user_ff_branch(
        &self,
        request: Request<UserFfBranchRequest>,
    ) -> Result<Response<UserFfBranchResponse>, Status> {
        sidecar::unary!(self, request, OperationServiceClient, user_ff_branch)
    }
}

/// Takes care of boilerplate that would instead be in the startup sequence.
pub fn operations_server(
    config: &Arc<Config>,
    shutdown_token: &CancellationToken,
    sidecar_servers: &Arc<sidecar::Servers>,
) -> OperationServiceServer<OperationsServiceImpl> {
    OperationServiceServer::new(OperationsServiceImpl {
        config: config.clone(),
        shutdown_token: shutdown_token.clone(),
        sidecar_servers: sidecar_servers.clone(),
    })
}
